import numpy as np
from matplotlib import pyplot as plt
import torch
import torchvision
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim

from .getData import *

device = 'cuda' if torch.cuda.is_available() else 'cpu'


def test(model, loder_test, print_val=True, rand_seed=0):
    model.eval()
    test_loss = 0
    correct = 0
    # torch.manual_seed(rand_seed)
    with torch.no_grad():
        for data, target in loder_test:
            data = data.to(device)
            target = target.to(device)
            # print(data.shape)
            output = model(data)
            # print(output.shape,target.shape)
            test_loss += F.cross_entropy(output, target, size_average=False).item()
            pred = output.data.max(1, keepdim=True)[1]
            correct += pred.eq(target.data.view_as(pred)).sum()
        test_loss /= len(loder_test.dataset)
        # test_losses.append(test_loss)
        if print_val:
            print('\nTest set: Avg. loss: {:.4f}, Accuracy: {}/{} ({:.2f}%)\n'.format(
                test_loss, correct, len(loder_test.dataset), 100. * correct / len(loder_test.dataset)))
        return float(100. * correct / len(loder_test.dataset))


def test_gba(model, loder_test, g_ba, print_val=True, rand_seed=0):
    # torch.manual_seed(rand_seed)
    model.eval()
    test_loss = 0
    correct = 0
    with torch.no_grad():
        for data, target in loder_test:
            data = data.to(device)
            target = target.to(device)
            data = g_ba(data)
            # print(data.shape)
            output = model(data)
            # print(output.shape,target.shape)
            test_loss += F.cross_entropy(output, target, size_average=False).item()
            pred = output.data.max(1, keepdim=True)[1]
            correct += pred.eq(target.data.view_as(pred)).sum()
        test_loss /= len(loder_test.dataset)
        # test_losses.append(test_loss)
        if print_val:
            print('\nTest set: Avg. loss: {:.4f}, Accuracy: {}/{} ({:.2f}%)\n'.format(
                test_loss, correct, len(loder_test.dataset), 100. * correct / len(loder_test.dataset)))
        return float(100. * correct / len(loder_test.dataset))


def val_plot(g_ba, base_model, atk_model=None):
    pn_ratio = [0, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1]
    deflist, nodeflist = [], []
    base_acc = test(base_model, normalMnist().loader)
    for eps in pn_ratio:
        if atk_model is None:
            x_atk = attackMnist(base_model, eps=eps)
        else:
            x_atk = attackMnist(atk_model, eps=eps)
        temp_def = test_gba(base_model, x_atk.loader, g_ba, print_val=False)
        temp_nodef = test(base_model, x_atk.loader, print_val=False)
        deflist.append(temp_def)
        nodeflist.append(temp_nodef)
    plt.figure(figsize=(15, 7))
    plt.plot(pn_ratio, deflist)
    plt.plot(pn_ratio, nodeflist)
    plt.plot(pn_ratio, np.ones_like(pn_ratio) * base_acc)
    plt.legend(['def', 'no def', 'base acc'])
    plt.ylabel('Accuracy')
    plt.xlabel('alpha ratio')
    plt.grid(which='major')
    plt.minorticks_on()
    plt.grid(which='minor', linestyle=':', linewidth='0.5', color='black')



import torch
from torch import nn
import torch.nn.functional as F


class cifar10_a(nn.Module):
    def __init__(self):
        super().__init__()
        self.conv1 = nn.Conv2d(in_channels=3, out_channels=64, kernel_size=5, stride=1, padding=2)
        self.conv2 = nn.Conv2d(in_channels=64, out_channels=128, kernel_size=5, stride=1, padding=2)
        self.conv3 = nn.Conv2d(in_channels=128, out_channels=256, kernel_size=5, stride=2)
        self.dense1 = nn.Linear(in_features=256 * 2 * 2, out_features=512)
        self.dense2 = nn.Linear(in_features=512, out_features=256)
        self.dense3 = nn.Linear(in_features=256, out_features=10)

    def forward(self, x):
        x = F.relu(self.conv1(x))
        x = F.max_pool2d(x, 2)
        x = F.relu(self.conv2(x))
        x = F.max_pool2d(x, 2)
        x = F.relu(self.conv3(x))
        x = F.dropout(x, 0.25)
        # x = x.view(-1,256*6*6)
        x = nn.Flatten()(x)
        x = F.relu(self.dense1(x))
        x = F.dropout(x, 0.5)
        x = self.dense2(x)
        x = F.relu(x)
        x = self.dense3(x)

        return x


class cifar10_b(nn.Module):
    def __init__(self):
        super().__init__()
        self.conv1 = nn.Conv2d(in_channels=3, out_channels=128, kernel_size=3, stride=1, padding=1)
        self.conv2 = nn.Conv2d(in_channels=128, out_channels=256, kernel_size=3, stride=1, padding=1)
        self.conv3 = nn.Conv2d(in_channels=256, out_channels=512, kernel_size=3, stride=2)
        self.dense1 = nn.Linear(in_features=512 * 3 * 3, out_features=512)
        self.dense2 = nn.Linear(in_features=512, out_features=10)

    def forward(self, x):
        x = F.relu(self.conv1(x))
        x = F.max_pool2d(x, 2)
        x = F.relu(self.conv2(x))
        x = F.max_pool2d(x, 2)
        x = F.relu(self.conv3(x))
        x = F.dropout(x, 0.25)
        # x = x.view(-1,256*6*6)
        x = nn.Flatten()(x)
        x = F.relu(self.dense1(x))
        x = F.dropout(x, 0.5)
        x = self.dense2(x)

        return x

def getmodel(model=cifar10_a(), load_path=None):
    device = 'cuda' if torch.cuda.is_available() else 'cpu'
    if not load_path is None:
        model.load_state_dict(torch.load(load_path, map_location=device))
    model.to(device)
    return model


import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
device

class generator_ab(nn.Module):
    def __init__(self):
        super(generator_ab,self).__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=4, kernel_size=2,stride=2,padding=0)
        self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=2,stride=2,padding=0)
        self.d3 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=2,stride=2,padding=1)
        self.d4 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=2,stride=2,padding=0)
        
        self.u1 = nn.Conv2d(in_channels=32,out_channels=16, kernel_size=2,padding=0)
        self.u2 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=2,padding=0)
        self.u3 = nn.Conv2d(in_channels=16,out_channels=4, kernel_size=2,padding=0)
        #self.u4 = nn.Conv2d(in_channels=4,out_channels=2, kernel_size=2,padding=0)
        self.output = nn.Conv2d(in_channels=8,out_channels=1,kernel_size=4,padding=0)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(d1), 0.2))
        d3 = F.instance_norm(F.leaky_relu(self.d3(d2), 0.2))
        d4 = F.instance_norm(F.leaky_relu(self.d4(d3), 0.2))
        
        x = F.upsample(d4, scale_factor=2)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.tanh(x)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.tanh(x)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x = F.upsample(u2, scale_factor=2)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.tanh(x)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
        x = F.upsample(u3, scale_factor=2)
        x = nn.ZeroPad2d((2,1,2,1))(x)
        x = F.tanh(self.output(x))
        return x

class generator_ba(nn.Module):
    def __init__(self):
        super(generator_ba,self).__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=4, kernel_size=2,stride=2,padding=0)
        self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=2,stride=2,padding=0)
        self.d3 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=2,stride=2,padding=1)
        self.d4 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=2,stride=2,padding=0)
        
        self.u1 = nn.Conv2d(in_channels=32,out_channels=16, kernel_size=2,padding=0)
        self.u2 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=2,padding=0)
        self.u3 = nn.Conv2d(in_channels=16,out_channels=4, kernel_size=2,padding=0)
        #self.u4 = nn.Conv2d(in_channels=4,out_channels=2, kernel_size=2,padding=0)
        self.output = nn.Conv2d(in_channels=8,out_channels=1,kernel_size=4,padding=0)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(d1), 0.2))
        d3 = F.instance_norm(F.leaky_relu(self.d3(d2), 0.2))
        d4 = F.instance_norm(F.leaky_relu(self.d4(d3), 0.2))
        
        x = F.upsample(d4, scale_factor=2)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.relu(x)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.relu(x)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x = F.upsample(u2, scale_factor=2)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.relu(x)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
        x = F.upsample(u3, scale_factor=2)
        x = nn.ZeroPad2d((2,1,2,1))(x)
        x = F.relu(self.output(x))
        return x

class discriminator(nn.Module):
    def __init__(self):
        super(discriminator,self).__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=4, kernel_size=2,stride=2,padding=0)
        self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=2,stride=2,padding=0)
        self.d3 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=2,stride=2,padding=1)
        self.d4 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=2,stride=2,padding=0)
        self.val = nn.Conv2d(in_channels=32,out_channels=1,kernel_size=4,stride=2,padding=1)

        
    def forward(self,x):
        x = F.leaky_relu(self.d1(x),0.2)
        x = F.instance_norm(F.leaky_relu(self.d2(x),0.2))
        x = F.instance_norm(F.leaky_relu(self.d3(x),0.2))
        x = F.instance_norm(F.leaky_relu(self.d4(x),0.2))
        x = self.val(x)
        #x = x.view(-1,1)
        x = F.sigmoid(x)
        return x
    
class generator_ba2(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=8, kernel_size=3,stride=1,padding=1)
        #self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.enmaxpool = nn.MaxPool2d(2)
        self.u1 = nn.Conv2d(in_channels=32,out_channels=32, kernel_size=3,padding=1)
        self.u2 = nn.Conv2d(in_channels=64,out_channels=16, kernel_size=3,padding=1)
        self.u3 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=3,padding=1)
        self.up1 = nn.Upsample(scale_factor=2)
        self.output = nn.Conv2d(in_channels=16,out_channels=1,kernel_size=3,padding=1)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        x = F.max_pool2d(d1,2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(x), 0.2))
        x = F.max_pool2d(d2,2)
        d3 = F.instance_norm(F.leaky_relu(self.d3(x), 0.2))
        encoder = self.enmaxpool(d3)
        #d4 = F.instance_norm(F.leaky_relu(self.d4(d3), 0.2))
        #x = F.upsample(d3, scale_factor=2)
        x = self.up1(encoder)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = self.up1(u1)
        #x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x  = self.up1(u2)
        #x = F.upsample(u2, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
       # x = F.upsample(u3, scale_factor=2)
        #x = nn.ZeroPad2d((2,1,2,1))(x)
        x = self.output(u3)
        x = F.relu(x)
        return x
    
class generator_ab2(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=8, kernel_size=3,stride=1,padding=1)
        #self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.enmaxpool = nn.MaxPool2d(2)
        self.u1 = nn.Conv2d(in_channels=32,out_channels=32, kernel_size=3,padding=1)
        self.u2 = nn.Conv2d(in_channels=64,out_channels=16, kernel_size=3,padding=1)
        self.u3 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=3,padding=1)
        self.up1 = nn.Upsample(scale_factor=2)
        self.output = nn.Conv2d(in_channels=16,out_channels=1,kernel_size=3,padding=1)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        x = F.max_pool2d(d1,2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(x), 0.2))
        x = F.max_pool2d(d2,2)
        d3 = F.instance_norm(F.leaky_relu(self.d3(x), 0.2))
        encoder = self.enmaxpool(d3)
        #d4 = F.instance_norm(F.leaky_relu(self.d4(d3), 0.2))
        #x = F.upsample(d3, scale_factor=2)
        x = self.up1(encoder)
        x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = self.up1(u1)
        #x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x  = self.up1(u2)
        #x = F.upsample(u2, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
       # x = F.upsample(u3, scale_factor=2)
        #x = nn.ZeroPad2d((2,1,2,1))(x)
        x = self.output(u3)
        x = F.tanh(x)
        return x
    
class discriminator2(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=1 , out_channels=4, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d4 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.val = nn.Conv2d(in_channels=32,out_channels=1,kernel_size=1,stride=1,padding=0)
        self.maxpool = nn.MaxPool2d(2)

        
    def forward(self,x):
        x = F.leaky_relu(self.d1(x),0.2)
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d2(x),0.2))
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d3(x),0.2))
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d4(x),0.2))
        x = self.maxpool(x)
        #return x
        x = self.val(x)
        #x = x.view(-1,1)
        x = F.sigmoid(x)
        return x
class gen_ab_cf(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=3 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.enmaxpool = nn.MaxPool2d(2)
        self.u1 = nn.Conv2d(in_channels=32,out_channels=32, kernel_size=3,padding=1)
        self.u2 = nn.Conv2d(in_channels=64,out_channels=16, kernel_size=3,padding=1)
        self.u3 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=3,padding=1)
        self.up1 = nn.Upsample(scale_factor=2)
        self.output = nn.Conv2d(in_channels=16,out_channels=3,kernel_size=3,padding=1)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        x = F.max_pool2d(d1,2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(x), 0.2))
        x = F.max_pool2d(d2,2)
        d3 = F.instance_norm(F.leaky_relu(self.d3(x), 0.2))
        encoder = self.enmaxpool(d3)
        x = self.up1(encoder)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = self.up1(u1)
        #x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x  = self.up1(u2)
        #x = F.upsample(u2, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
       # x = F.upsample(u3, scale_factor=2)
        #x = nn.ZeroPad2d((2,1,2,1))(x)
        x = self.output(u3)
        x = F.tanh(x)
        return x

class gen_ba_cf(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=3 , out_channels=8, kernel_size=3,stride=1,padding=1)
        #self.d2 = nn.Conv2d(in_channels=4 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.enmaxpool = nn.MaxPool2d(2)
        self.u1 = nn.Conv2d(in_channels=32,out_channels=32, kernel_size=3,padding=1)
        self.u2 = nn.Conv2d(in_channels=64,out_channels=16, kernel_size=3,padding=1)
        self.u3 = nn.Conv2d(in_channels=32,out_channels=8, kernel_size=3,padding=1)
        self.up1 = nn.Upsample(scale_factor=2)
        self.output = nn.Conv2d(in_channels=16,out_channels=3,kernel_size=3,padding=1)
        
    def forward(self,x):
        d1 = F.leaky_relu(self.d1(x), 0.2)
        x = F.max_pool2d(d1,2)
        d2 = F.instance_norm(F.leaky_relu(self.d2(x), 0.2))
        x = F.max_pool2d(d2,2)
        d3 = F.instance_norm(F.leaky_relu(self.d3(x), 0.2))
        encoder = self.enmaxpool(d3)
        #d4 = F.instance_norm(F.leaky_relu(self.d4(d3), 0.2))
        #x = F.upsample(d3, scale_factor=2)
        x = self.up1(encoder)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u1(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u1 = torch.cat((x,d3),1)
        x = self.up1(u1)
        #x = F.upsample(u1, scale_factor=2)
        x = self.u2(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u2 = torch.cat((x,d2),1)
        x  = self.up1(u2)
        #x = F.upsample(u2, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        x = self.u3(x)
        x = F.leaky_relu(x,0.2)
        x = F.instance_norm(x)
        u3 = torch.cat((x,d1),1)
        #x = F.upsample(x, scale_factor=2)
        #x = nn.ZeroPad2d((1,0,1,0))(x)
        #x = self.u4(x)
       # x = F.upsample(u3, scale_factor=2)
        #x = nn.ZeroPad2d((2,1,2,1))(x)
        x = self.output(u3)
        x = F.relu(x)
        return x

class dis_cf(nn.Module):
    def __init__(self):
        super().__init__()
        self.d1 = nn.Conv2d(in_channels=3 , out_channels=8, kernel_size=3,stride=1,padding=1)
        self.d2 = nn.Conv2d(in_channels=8 , out_channels=16, kernel_size=3,stride=1,padding=1)
        self.d3 = nn.Conv2d(in_channels=16 , out_channels=32, kernel_size=3,stride=1,padding=1)
        self.d4 = nn.Conv2d(in_channels=32 , out_channels=64, kernel_size=3,stride=1,padding=1)
        self.d5 = nn.Conv2d(in_channels=64 , out_channels=128, kernel_size=3,stride=1,padding=1)
        self.val = nn.Conv2d(in_channels=128,out_channels=1,kernel_size=1,stride=1,padding=0)
        self.maxpool = nn.MaxPool2d(2)

        
    def forward(self,x):
        x = F.leaky_relu(self.d1(x),0.2)
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d2(x),0.2))
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d3(x),0.2))
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d4(x),0.2))
        x = self.maxpool(x)
        x = F.instance_norm(F.leaky_relu(self.d5(x),0.2))
        x = self.maxpool(x)
        #return x
        x = self.val(x)
        #x = x.view(-1,1)
        x = F.sigmoid(x)
        return x